import events
from pygame.sprite import Sprite
from pygame import Rect
import pygame
from utils import *
import math

DRAGTWIST_RATIO = 80
DRAGSTRETCH_RATIO = 20

def ChangeAndDirtyAttr( attrName ):
	def Setter(self, val):
		print 'setting ', attrName, val
		if val == self.__dict__[attrName]:
			return
		self.__dict__[attrName] = val
		self._imageDirty = True
	def Getter(self):
		return self.__dict__[attrName]

	return property(Getter, Setter)

class Focusable:
	def __init__(self):
		self._focused = False
		self._imageDirty = True
	focused = ChangeAndDirtyAttr('_focused')
class Dragable:
	def __init__(self):
		self._dragging = False
		self.dragStartPoint = None
		self._imageDirty = True
	dragging = ChangeAndDirtyAttr('_dragging')

class Dot(Sprite):
	red = 0
	def __init__(self, radius):
		Sprite.__init__(self)
		self.radius = radius
		self.image = TransSurface( self.radius*2, self.radius*2 )
		if DEBUG:
			color = (Dot.red,Dot.red,255,255)
			Dot.red += 10
		else:
			color = (255,255,255,255)
		pygame.draw.circle( self.image, color, 
		                    (self.radius,self.radius), self.radius )
		self.rect = self.image.get_rect()
		#print 'self.rect', self.rect
	#def getRect(self):
	#	print 'getting dot rect'
	#	import traceback
	#	traceback.print_stack()
	#	return self.__dict__['_rect']
	#def setRect(self, val):
	#	print 'setting dot rect'
	#	self.__dict__['rect'] = val
	#rect = property( getRect, setRect )

class ElasticBand(Sprite, Focusable, Dragable):
	def __init__(self, radius=6, width=20):
		Sprite.__init__(self)
		Focusable.__init__(self)
		Dragable.__init__(self)
		events.AddListener(self)

		self.radius = radius
		self.width = width

		self.stretch = 40
		self.stretchFloor = 40
		self.stretchCeiling = 200

		self.twist = 0
		self.twistFloor = 0
		self.twistCeiling = 3

		self.dots = [
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		            ]

		self.active = True

		self._cachedImage = None
		self._newDot0Center = None

		print 'last 4'
		self.dots[0].rect.center = (self.radius,self.radius)
		self.CalcDotPositions()
		self.CalcOriginPoint()
		print 'done init'

	
	def CalcOriginPoint(self):
		self.originPoint = HalfwayBetween( self.dots[0].rect.center,
		                                   self.dots[1].rect.center )
	def SetOriginPoint(self, pos):
		self.CalcOriginPoint()
		change = VectorSubtract( pos, self.originPoint )
		d0rect = self.dots[0].rect
		self._newDot0Center = VectorAdd( d0rect.center, change )

	def GetImage(self):
		if not self._imageDirty:
			return self._cachedImage

		self.CalcDotPositions()

		print self.rect.size
		newImg = TransSurface( abs(self.rect.width), 
		                       abs(self.rect.height) )

		for dot in ds:
			#print 'blitting dot at', dot.rect, 'on', self.rect
			newImg.blit( dot.image, dot.rect )

		if DEBUG:
			if self.focused:
				color = (255,100,100)
			else:
				color = (200,0,0)
			pygame.draw.rect( newImg, color, self.rect, 2 )
			pygame.draw.circle( newImg, color, self.originPoint, 3 )

		self._cachedImage = newImg
		self._imageDirty = False
		return newImg

	def SetImage(self, value):
		print "oh really fool?"

	#image = property(GetImage, SetImage)

	def CalcDotPositions(self):
		ds = self.dots
		minX = ds[0].rect.left
		minY = ds[0].rect.top
		maxX = ds[0].rect.right
		maxY = ds[0].rect.bottom
		pairs = zip( ds[::2], ds[1::2] )
		stretchStep = self.stretch/len(pairs)
		twistStep = self.twist/len(pairs)

		for i, pair in enumerate(pairs):
			pair[0].rect.center = ds[0].rect.center

			pair[1].rect.centerx = pair[0].rect.centerx
			pair[1].rect.centery = pair[0].rect.centery + self.width

			pair[0].rect.centerx += i*stretchStep
			pair[1].rect.centerx += i*stretchStep

			thisTwist = math.sin(math.pi*twistStep*i)*self.width
			thisTwist = abs(thisTwist)
			#print 'percentage pi *', twistStep, i
			#print 'percentage', math.sin(math.pi*twistStep*i)
			#print 'thisTwist', thisTwist
			pair[0].rect.centery += thisTwist
			pair[1].rect.centery -= thisTwist

			for dot in pair:
				minX = min( dot.rect.left, minX )
				maxX = max( dot.rect.right, maxX )
				minY = min( dot.rect.top, minY )
				maxY = max( dot.rect.bottom, maxY )
		self.CalcOriginPoint()
		self.rect = Rect( minX, minY, (maxX-minX), (maxY-minY) )
		print 'new rect', self.rect
		for d in self.dots:
			print 'dot', d.rect


	def StretchMore( self, furtherStretch ):
		self.stretch += furtherStretch/DRAGSTRETCH_RATIO
		self.stretch = Clamp( self.stretch, 
		                      self.stretchFloor, self.stretchCeiling )
		twistAmt = (self.stretch - self.stretchFloor) /\
		           (self.stretchCeiling - self.stretchFloor)
		self.twist = (self.twistCeiling - self.twistFloor) * twistAmt
		
	def update(self, timeChange=1):
		if self._newDot0Center:
			self.dots[0].rect.center = self._newDot0Center
			self._newDot0Center = None
			self._imageDirty = True
		if not self.dragging:
			self.update_UnwindStep(timeChange)

		if not self._imageDirty:
			return

		print 'image was dirty. calcing'
		self.CalcDotPositions()
	
	def update_UnwindStep(self, timeChange):
		if not self.stretch == self.stretchFloor:
			self.StretchMore( -2.8 )
			self._imageDirty = True

	def On_MouseDown(self, event):
		print 'on mouse down'
		if not self.rect.collidepoint( event.pos ):
			self.focused = False
			return

		self.focused = True
		self.dragging = True
		self.dragStartPoint = event.pos

	def On_MouseUp(self, event):
		print 'on mouse up'
		if not self.focused:
			return

		self.dragging = False
		

	def On_MouseMotion(self, event):
		#print 'handling mousemouion'
		if not self.focused and self.rect.collidepoint( event.pos ):
			self.focused = True
		if not self.dragging:
			return

		self._imageDirty = True
		currentDragDistance = Distance( event.pos, self.dragStartPoint )
		originToDragStart = Distance( self.originPoint, self.dragStartPoint )
		originToDragPos = Distance( self.originPoint, event.pos )
		if originToDragPos < originToDragStart:
			#reverse it if dragging towards the origin
			currentDragDistance *= -1
		self.StretchMore( currentDragDistance )

			
		

class Marble(Sprite, Focusable, Dragable):
	def __init__(self, elasticBand):
		Sprite.__init__(self)
		Focusable.__init__(self)
		Dragable.__init__(self)
		self.elasticBand = elasticBand
		self.velocity = [1,0]
		events.AddListener(self)
		self._cachedAngle = 0

		self.radius = 50
		self.image = TransSurface( self.radius*2, self.radius*2 )
		color = (255,255,255,255)
		pygame.draw.circle( self.image, color, 
		                    (self.radius,self.radius), self.radius )

		self.rect = self.image.get_rect()

	def update( self, timeChange ):
		self.elasticBand.SetOriginPoint( self.rect.center )
		self.elasticBand.update( timeChange )
		self.update_AngleDots()
		self.rect.move_ip( *self.velocity )
		print 'updated marble.  new rect', self.rect

		
	def update_AngleDots(self):
		angle = GetAngle( VectorMultiply( self.velocity,(-1,-1)) )
		print 'angle', angle, angle*180/math.pi
		#if round(angle,8) == round(self._cachedAngle,8):
			#return
		self._cachedAngle = angle

		for dot in self.elasticBand.dots:
			x,y = dot.rect.center
			newX = (x*math.cos(angle))+0 #problem!
			newY = (x*math.sin(angle))+self.elasticBand.width/2
			dot.rect.center = (newX, newY)

	def On_MouseDown(self, event):
		print 'on mouse down'
		if not self.rect.collidepoint( event.pos ):
			self.focused = False
			return

		self.focused = True
		self.dragging = True
		self.dragStartPoint = event.pos

	def On_MouseUp(self, event):
		print 'on mouse up'
		if not self.focused:
			return

		self.dragging = False

	def On_MouseMotion(self, event):
		#print 'handling mousemouion'
		if not self.focused and self.rect.collidepoint( event.pos ):
			self.focused = True
		if not self.dragging:
			return

		self._imageDirty = True
		currentDragDistance = Distance( event.pos, self.dragStartPoint )
		centerToDragStart = Distance( self.rect.center, self.dragStartPoint )
		centerToDragPos = Distance( self.rect.center, event.pos )
		if centerToDragPos < centerToDragStart:
			#reverse it if dragging towards the center
			currentDragDistance *= -1
		self.elasticBand.StretchMore( currentDragDistance )

