import events
import pygame
from pygame.locals import *
import data
from game_screen import GameScreen
#from intro_screen import IntroScreen

class SimpleSprite(pygame.sprite.Sprite):
	def __init__(self, name, pos):
		pygame.sprite.Sprite.__init__(self)

		self.image = data.pngs[name]

		self.rect = self.image.get_rect()
		self.rect.topleft = pos

class SimpleButton(pygame.sprite.Sprite):
	def __init__(self, name, pos, callback):
		pygame.sprite.Sprite.__init__(self)

		self.name = name
		self.callback = callback

		self.upImage = data.pngs[name+'_up']
		self.downImage = data.pngs[name+'_down']

		self.image = self.upImage
		self.rect = self.image.get_rect()
		self.rect.center = pos

	def Press(self):
		self.image = self.downImage
	def Release(self):
		self.image = self.upImage

class MainMenu:
	def __init__(self, screen, display):
		self.screen = screen
		self.display = display

		self.isDone = False
		self.isStarted = False
		self.nextDisplayer = GameScreen
		self.spriteGroup = pygame.sprite.RenderUpdates()
		print dir(data)
		self.bg = data.pngs['menu_background']

		cx,cy = screen.get_rect().center
		self.choices = [
		                SimpleButton( 'menu_start', (cx,cy),
		                              self.Callback_Level ),
		                #SimpleButton( 'menu_intro', (cx,cy+80), 
		                              #self.Callback_Intro ),
		               ]

		self.curChoice = 0
		for choice in self.choices:
			self.spriteGroup.add( choice )

		self.cursor = SimpleSprite( 'blob', (200,0) )
		self.cursor.rect.centery = self.choices[0].rect.centery
		self.spriteGroup.add( self.cursor )


	def Start( self ):
		events.AddListener( self )
		self.isDone = False
		self.isStarted = True
		self.screen.blit( self.bg, (0,0) )
		self.display.flip()

	def Finish( self ):
		self.isDone = True
		self.isStarted = False
		events.RemoveListener( self )

	#def Callback_Intro(self):
		#self.nextDisplayer = IntroScreen
		#self.Finish()

	def Callback_Level(self):
		self.nextDisplayer = GameScreen
		self.Finish()

	def On_MouseDown(self, event):
		#should be doing this with pointcollide on a spritegroup...
		#but i'm lazy so...
		for s in self.choices:
			if s.rect.collidepoint( event.pos ):
				s.Press()

	def On_MouseUp(self, event):
		for s in self.choices:
			if s.rect.collidepoint( event.pos ) and \
			   s.image == s.downImage:
				s.callback()
			s.Release()


	def On_KeyDown(self, event):
		if event.key in (K_RETURN, K_SPACE):
			self.choices[self.curChoice].Press()
		elif event.key in (K_UP, K_RIGHT):
			self.curChoice += 1
		elif event.key in (K_DOWN, K_LEFT):
			self.curChoice -= 1

		self.curChoice %= len( self.choices )
		self.UpdateCursor()

	def On_KeyUp(self, event):
		if event.key not in (K_RETURN, K_SPACE):
			return

		self.choices[self.curChoice].Release()
		self.choices[self.curChoice].callback()

	def UpdateCursor(self):
		#print 'moving cursor'
		self.cursor.rect.centery = \
		                      self.choices[self.curChoice].rect.centery

	def Tick(self, timeChange):
		if not self.isStarted:
			#print 'starting'
			self.Start()
		#clear
		self.spriteGroup.clear( self.screen, self.bg )

		#update
		events.ConsumeEventQueue()
		self.spriteGroup.update( timeChange )
		events.ConsumeEventQueue()

		#draw
		changedRects = self.spriteGroup.draw(self.screen)
		self.display.update( changedRects )

